#include <ESP8266WiFi.h>

#define LED_R 13 
#define LED_G 14
#define LED_B 16 
#define SPEED_SLOW 4
#define SPEED_MID 16
#define SPEED_FAST 32
#define LED_R_HANDYCAP 1.7

WiFiServer server(80); // Webserver
int i_pwr_r, i_pwr_g, i_pwr_b; // IST Werte
int s_pwr_r, s_pwr_g, s_pwr_b; // SOLL Werte
int pwr_mode; // Fade Modus
os_timer_t ledTimer; // Timer
WiFiClient client; // WLAN Client Object
bool fadeDone;
String header; // HTTP Header


void ledTimerCallback(void *pArg) {
  if (fadeDone == false) {
    if (pwr_mode == 0) {
      i_pwr_r = s_pwr_r ;
      i_pwr_g = s_pwr_g ;
      i_pwr_b = s_pwr_b ;    
    } else if (pwr_mode == 1) {
      fadeLed(SPEED_SLOW);
    } else if (pwr_mode == 2) {
      fadeLed(SPEED_MID);
    } else if (pwr_mode == 3) {
      fadeLed(SPEED_FAST);
    }
    analogWrite(LED_R, i_pwr_r);
    analogWrite(LED_G, i_pwr_g);
    analogWrite(LED_B, i_pwr_b);   
  }
  
  if ((i_pwr_r == s_pwr_r) && (i_pwr_g == s_pwr_g) && (i_pwr_b == s_pwr_b) && (fadeDone == false)) {
    fadeDone = true;
  }
} // End of ledTimerCallback

              
void setup() {
  pinMode(LED_R, OUTPUT); 
  pinMode(LED_G, OUTPUT); 
  pinMode(LED_B, OUTPUT); 

  i_pwr_r = 0;
  i_pwr_g = 0;
  i_pwr_b = 0;
  s_pwr_r = 0;
  s_pwr_g = 0;
  s_pwr_b = 0;
  pwr_mode = 0;
  fadeDone = true;
  
  Serial.begin(115200);

  IPAddress ip(192, 168, 1, XXX);
  IPAddress gateway(192, 168, 1, YYY); 
  IPAddress subnet(255, 255, 255, 0); 
  WiFi.config(ip, gateway, subnet);
  WiFi.begin("SSID", "PASSWORD");

  Serial.print("Connecting");
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println();

  Serial.print("Connected, IP address: ");
  Serial.println(WiFi.localIP());  
  server.begin();
  Serial.println("Waiting for clients");
  Serial.println("");

  os_timer_setfn(&ledTimer, ledTimerCallback, NULL);
  os_timer_arm(&ledTimer, 10, true);
}


void loop() {
  client = server.available();   
  if (client) {                             
    Serial.println("New Client.");
    String currentLine = "";                
    while (client.connected()) {
      if (client.available()) {
        char c = client.read();            
        Serial.write(c);                    
        header += c;
        if (c == '\n') {                    
          if (currentLine.length() == 0) {
            int start = header.indexOf("GET /");
            int ende = header.indexOf(" HTTP/");            
            if (start >= 0) {
              String parameter = header.substring(start+4,ende);
              
              if (getValue(parameter, '/', 1) == "r") {
                s_pwr_r = getValue(parameter, '/', 2).toInt() / LED_R_HANDYCAP;
              }
              if (getValue(parameter, '/', 3) == "g") {
                s_pwr_g = getValue(parameter, '/', 4).toInt();
              }
              if (getValue(parameter, '/', 5) == "b") {
                s_pwr_b = getValue(parameter, '/', 6).toInt();
              }
              if (getValue(parameter, '/', 7) == "m") {
                pwr_mode = getValue(parameter, '/', 8).toInt();
              }
              fadeDone = false;

              Serial.print("Fade to:");
              Serial.print("R=");Serial.print(s_pwr_r);Serial.print(";");
              Serial.print("G=");Serial.print(s_pwr_g);Serial.print(";");
              Serial.print("B=");Serial.print(s_pwr_b);Serial.print(";");
              Serial.print("M=");Serial.println(pwr_mode);

              while (fadeDone == false) {
                delay(10);
              }

              htmlPrintResponse();
              htmlPrintWebpage();              
            }
                   
            break;
          } else { 
            currentLine = "";
          }
        } else if (c != '\r') {
          currentLine += c;    
        }
      } // if (client.available()) {
    } // while (client.connected()) {
    header = "";
    client.stop();

    Serial.println("Client disconnected.");
    Serial.println("");
  }
}

String getValue(String data, char separator, int index) {
  int found = 0;
  int strIndex[] = {0, -1};
  int maxIndex = data.length()-1;

  for(int i=0; i<=maxIndex && found<=index; i++){
    if(data.charAt(i)==separator || i==maxIndex){
        found++;
        strIndex[0] = strIndex[1]+1;
        strIndex[1] = (i == maxIndex) ? i+1 : i;
    }
  }

  return found>index ? data.substring(strIndex[0], strIndex[1]) : "";
} // getValue

void fadeLed(int speed) {
  if (i_pwr_r > s_pwr_r+speed) {
    i_pwr_r = i_pwr_r - (speed/2);
  } else if (i_pwr_r < s_pwr_r-speed) {
    i_pwr_r = i_pwr_r + (speed/2);
  } else {
    i_pwr_r = s_pwr_r;
  }
  
  if (i_pwr_g > s_pwr_g+speed) {
    i_pwr_g = i_pwr_g - speed;
  } else if (i_pwr_g < s_pwr_g-speed) {
    i_pwr_g = i_pwr_g + speed;
  } else {
    i_pwr_g = s_pwr_g;
  }

  if (i_pwr_b > s_pwr_b+speed) {
    i_pwr_b = i_pwr_b - speed;
  } else if (i_pwr_b < s_pwr_b-speed) {
    i_pwr_b = i_pwr_b + speed;
  } else {
    i_pwr_b = s_pwr_b;
  }
} // fadeLed

void htmlPrintResponse() {
  client.println("HTTP/1.1 200 OK");
  client.println("Content-type:text/html");
  client.println("Connection: close");
  client.println();
} // htmlPrintResponse

void htmlPrintWebpage() {
  client.println("<!DOCTYPE html><html>");
  client.println("<head><meta name=\"viewport\" content=\"width=device-width, initial-scale=1\">");
  client.println("<link rel=\"icon\" href=\"data:,\">");
  client.println("<body><h1>RGB Flood Web Server</h1>");
  client.println("<p>Use get parameter r, g, b for colors</p>");
  client.println("<p>Use get parameter m for fade-mode</p>");
  client.println("<p>Exampe: /r/1024/g/500/b/200/m/1</p>");
  client.println("</body></html>");
  client.println();
} // htmlPrintWebpage
